'use strict';

var describe = require('mocha').describe;
var it = require('mocha').it;
var expect = require('chai').expect;
var codemo = require('./index');
var path = require('path');

describe('codemo', function () {
  it('should generate example from a file', function () {
    return codemo.processFile(path.resolve(__dirname, './test/hello-world-example.js')).then(function (actual) {
      expect(actual).to.eq(['console.log(\'Hello world!\')', '//> Hello world!', ''].join('\n'));
    });
  });

  it('should work with requires', function () {
    return codemo.processFile(path.resolve(__dirname, './test/require-example/example.js')).then(function (actual) {
      expect(actual).to.eq(['\'use strict\'', 'var fooBar = require(\'./index\')', 'console.log(fooBar)', '//> Hello world!', ''].join('\n'));
    });
  });

  it('should work with imports', function () {
    return codemo.processFile(path.resolve(__dirname, './test/import-example/example.js'), { es6: true }).then(function (actual) {
      expect(actual).to.eq(["import fooBar from './index'", 'console.log(fooBar)', '//> Hello world!', ''].join('\n'));
    });
  });

  it('should generate example from an es6 file', function () {
    return codemo.processFile(path.resolve(__dirname, './test/hello-world-example.es6.js'), { es6: true }).then(function (actual) {
      expect(actual).to.eq(["'use strict'", "var s = 'Hello world!'", 'console.log(s)', '//> Hello world!', ''].join('\n'));
    });
  });
});